#include "csprite.h"
#include <math.h>


/*

Pretty much the same as the HS Fortuna example.

In stead of UV cooordinates for the texture, I'm using ST coordinates...
Pretty much the same, I'm just more used to ST coordinates...

While UV coordiantes are 0 to pixelWidth, ST coordinates are 0.0 to 1.0
independent of the actual pixel width...

You can always convert with (startTextureX/TextureWidth, endTextureX/TextureWidth).
For example, your Texture is 256x256 and you want to use a 32x48 square starting at 20,80.
S1 = 20/256, S2 = (20+32)/256, T1 = 80/256, T2 = (80+48)/256...

Then just draw a triangle Strip like this:

		  1                  3
		Pos1 ------------- Pos2
		 +				  /	+
		 +	 /  SPRITE	/	+
		 + /				+
		Pos3 ------------- Pos4
         2					4

First triangle is Pos1,Pos3,Pos2
Second triangle is Pos2,Pos3,Pos4
*/	


// Initialize sprite.
CSprite::CSprite(int xPos, int yPos, int width, int height)
{
	m_width = width;
	m_height = height;
	fHalfW = (float)width/2.0;
	fHalfH = (float)height/2.0;
	m_xPos = xPos;
	m_yPos = yPos;
	m_isVisible = true;
	m_Red = 0x80;
	m_Green = 0x80;
	m_Blue = 0x80;
	m_s1 = 0.0;
	m_s2 = 1.0;
	m_t1 = 0.0;
	m_t2 = 1.0;
	m_Angle = 0.0;
	setPos(m_xPos, m_yPos);
}

// Clean up.
CSprite::~CSprite()
{
}

// Set Position.
// When we set the position, we also calcualte the four corners depending on size and rotation.
// This is the same as the HS Fortuna example.
void CSprite::setPos(int xPos, int yPos) {
	m_xPos = xPos;
	m_yPos = yPos;

	//
	float cosi = cosf(m_Angle);
	float sini = sinf(m_Angle);
		
	// Untransformed coordinates
	// tl = top left
	// bl = bottom left
	// tr = top right
	// br = bottom right
	float _tlX = -fHalfW, _tlY = -fHalfH;
	float _blX = -fHalfW, _blY =  fHalfH;
	float _trX =  fHalfW, _trY = -fHalfH;
	float _brX =  fHalfW, _brY =  fHalfH;

	// Rotate the coordinates
	float tlX  = _tlX * cosi - _tlY * sini;
	float tlY  = _tlY * cosi + _tlX * sini;

	float blX  = _blX * cosi - _blY * sini;
	float blY  = _blY * cosi + _blX * sini;

	float trX  = _trX * cosi - _trY * sini;
	float tr_Y  = _trY * cosi + _trX * sini;

	float brX = _brX * cosi - _brY * sini;
	float brY = _brY * cosi + _brX * sini;

	m_xPos1 = m_xPos + (int)(tlX + 0.5);
	m_yPos1 = m_yPos + (int)(tlY + 0.5);

	m_xPos2 = m_xPos + (int)(trX + 0.5);
	m_yPos2 = m_yPos + (int)(tr_Y + 0.5);

	m_xPos3 = m_xPos + (int)(blX + 0.5);
	m_yPos3 = m_yPos + (int)(blY + 0.5);

	m_xPos4 = m_xPos + (int)(brX + 0.5);
	m_yPos4 = m_yPos + (int)(brY + 0.5);

}

// We can also set all four coordinates of the sprite, this comes in handy if you don't want square sprites.
// This is being used in the example...
void CSprite::setPosAll(int xPos1, int yPos1, int xPos2, int yPos2, int xPos3, int yPos3, int xPos4, int yPos4) {
	m_xPos1 = xPos1;
	m_yPos1 = yPos1;
	m_xPos2 = xPos2;
	m_yPos2 = yPos2;
	m_xPos3 = xPos3;
	m_yPos3 = yPos3;
	m_xPos4 = xPos4;
	m_yPos4 = yPos4;
}

// Rotation Angle
void CSprite::setAngle(float Angle) {
	m_Angle = Angle;
	setPos(m_xPos, m_yPos);
}

// Incremenat rotation Angle
void CSprite::addAngle(float Angle) {
	m_Angle += Angle;
	setPos(m_xPos, m_yPos);
}


// Move sprite x, y positions.
void CSprite::movePos(int xPos, int yPos) {
	m_xPos += xPos;
	m_yPos += yPos;
	setPos(m_xPos, m_yPos);
}

// Which part of the texture to use.
// Rememeber ST uses 0.0 to 1.0 in stead of pixel coordinates...
void CSprite::setTexturePos(float s1, float s2, float t1, float t2) {
	m_s1 = s1;
	m_s2 = s2;
	m_t1 = t1;
	m_t2 = t2;
}

// Set sprite visibility.
void CSprite::setVisible(bool isVisible) {
	m_isVisible = isVisible;
}

// Is the sprite visible ???
bool CSprite::getVisible(void) {
	return m_isVisible;
}

// Get X position center of Sprite
int CSprite::getXPos(void) {
	return m_xPos;
}

// Get Y position center of Sprite
int CSprite::getYPos(void) {
	return m_yPos;
}

// Get X position top left corner.
int CSprite::getXPos1(void) {
	return m_xPos1;
}

// Get Y position top left corner.
int CSprite::getYPos1(void) {
	return m_yPos1;
}

// Get X position top right corner.
int CSprite::getXPos2(void) {
	return m_xPos2;
}

// Get Y position top right corner.
int CSprite::getYPos2(void) {
	return m_yPos2;
}

// Get X position bottom left corner.
int CSprite::getXPos3(void) {
	return m_xPos3;
}

// Get Y position bottom left corner.
int CSprite::getYPos3(void) {
	return m_yPos3;
}

// Get X position bottom right corner.
int CSprite::getXPos4(void) {
	return m_xPos4;
}

// Get Y position bottom right corner.
int CSprite::getYPos4(void) {
	return m_yPos4;
}

// Add Sprite to primitive memory.
void CSprite::addSpriteToMem(char *& pMem, int Zvalue) {

	// Only render visible sprites.
	if (!m_isVisible) {
		return;
	}

	// Since we are using Triangle Strips, we can only write one sprite at a time...
	// Previously we could use one GIFTag header for all our square sprites, this is not the case anymore.
	AddGIFTag(pMem, 1);
	addSpriteToGIFTag(pMem, Zvalue);
}


// Just add the GIF Tag to the beginning of the allocated Memory.
void CSprite::AddGIFTag(char *& pMem, int numSprites) {

		// Add a GIF Tag 
		sps2GIFTag_t * pGIFTag = (sps2GIFTag_t *) pMem;
 
	  	// Create the GIFTag  
  		pGIFTag->i128=0;       			// Blank it out.
	  	pGIFTag->s.NLOOP=numSprites;   	// How many sprites we are rendering.
  		pGIFTag->s.PRE=1;      			// Send our PRIM value to the GIF via this GIFTag.
	  	pGIFTag->s.EOP=1;      			// This is the end of the packets, no more GIFTags are to be expected after the sprites.
		pGIFTag->s.PRIM = GS_PRIM(PRIM_TRI_STRIP, 
							PRIM_IIP_FLAT, 
							PRIM_TME_ON,
							PRIM_FGE_OFF, 
							PRIM_ABE_ON, 
							PRIM_AA1_OFF, 
							PRIM_FST_STQ, 
							PRIM_CTXT_CONTEXT1, 
							PRIM_FIX_NOFIXDDA);
	  	pGIFTag->s.FLG=GIF_FLG_PACKED;	// Use packed format for data.

	 	pGIFTag->s.NREG = 9; 
		pGIFTag->s.REGS =    GIF_REG_RGBAQ 		| \
							(GIF_REG_ST << 4) 	| \
							(GIF_REG_XYZ2 << 8) | \
							(GIF_REG_ST << 12) 	| \
							(GIF_REG_XYZ2 << 16)| \
							(GIF_REG_ST << 20) 	| \
							(GIF_REG_XYZ2 << 24)| \
							(GIF_REG_ST << 28) 	| \
							(((uint64)GIF_REG_XYZ2) << 32);
		pGIFTag++;

		// Update memory pointer.
		pMem = (char *)pGIFTag;
}

// Render sprite.
void CSprite::addSpriteToGIFTag(char *& pMem, int Zvalue) {

	// pGIFReg is now pointing to the first memory position.
  	sps2GIFPackedRegister_t * pGIFReg = (sps2GIFPackedRegister_t *) pMem;

	AddColour(pGIFReg, m_Red, m_Green, m_Blue, 0x80);

	// Upper Left Corner.
	AddSTQ(pGIFReg, m_s1, m_t1, 1.0f);
	AddVertex(pGIFReg, m_xPos1, m_yPos1, Zvalue);

	// Upper Right Corner.
	AddSTQ(pGIFReg, m_s1, m_t2, 1.0f);
	AddVertex(pGIFReg, m_xPos3, m_yPos3, Zvalue);

	// Bottom Left Corner.
	AddSTQ(pGIFReg, m_s2, m_t1, 1.0f);
	AddVertex(pGIFReg, m_xPos2, m_yPos2, Zvalue);

	// Bottom Right Corner.
	AddSTQ(pGIFReg, m_s2, m_t2, 1.0f);
	AddVertex(pGIFReg, m_xPos4, m_yPos4, Zvalue);

	// Update memory pointer.
	pMem = (char *)pGIFReg;
}