#ifndef __TEXTURE_H__
#define __TEXTURE_H__

#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2regstructs.h>
#include <assert.h>
#include "PS2Defines.h"
#include "AddRegisters.h"

// All the following is the layout of a bitmap file header (the bit before
// the actual image data). The __attribute__((packed)) tells the compiler
// not to insert any annoying padding. (If it was padded we wouldn't be
// able to read it in in one go easily)
#define BITMAP_ID      0x4D42

struct BitmapFileHeader
{
	uint16  bfType;
    uint32  bfSize;
    uint16  bfReserved1;
    uint16  bfReserved2;
    uint32  bfOffBits;
} __attribute__((packed));

struct BitmapInfoHeader
{
	uint32	biSize;
	int32	biWidth;
	int32	biHeight;
	uint16	biPlanes;
	uint16	biBitCount;
	uint32	biCompression;
	uint32	biSizeImage;
	int32	biXPelsPerMeter;
	int32	biYPelsPerMeter;
	uint32	biClrUsed;
	uint32	biClrImportant;
} __attribute__((packed));

struct BitmapFile
{
	BitmapFileHeader  bitmapfileheader;
	BitmapInfoHeader  bitmapinfoheader;
} __attribute__((packed));

class CTexture
{
public:
	CTexture(int iSPS2Desc);
	~CTexture();

	bool LoadBitmap(const char * strFilename, bool bBlackTransparent = true, bool bRedAsAlpha = false);
	void Upload(uint32 uGSPage);

private:
	bool AllocateMemory(int Memory_Size);
	void BuildDMA(uint32 uGSPage);
	char * m_pMemory;
	char * m_pDMAMem;
	int m_iClutOffset;
	int m_iWidth, m_iHeight;
	int m_l2Width, m_l2Height;
	bool m_bHasCLUT;
	bool m_bHasDMA;
	int m_iSPS2Desc;
	sps2Memory_t * m_pMemCached;
	uint32 m_uGSPage;

};

#endif